package opam-repository

  1. Overview
  2. Docs

Module OpamGitSource

Git repository backend (based on OpamVCS)

Sourceval env : unit -> string array

Returns OpamProcess.default_env + some git specific environment variables used to make git calls more reproducible. Note however that it cannot be used if you need values from the global git configs (e.g. /etc/gitconfig or ~/.gitconfig)