Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
OpamRepositoryConfigSourceConfiguration options for the repository lib (record, global reference, setter, initialisation)
Toggles parsing of the tool's output to detect errors (curl returns 0 on a 404)
type t = {download_tool : (OpamTypes.arg list * dl_tool_kind) Lazy.t;validation_hook : OpamTypes.arg list option;retries : int;force_checksums : bool option;repo_tarring : bool;}type 'a options_fun =
?download_tool:(OpamTypes.arg list * dl_tool_kind) Lazy.t ->
?validation_hook:OpamTypes.arg list option ->
?retries:int ->
?force_checksums:bool option ->
?repo_tarring:bool ->
'ainclude OpamStd.Config.Sig
with type t := t
and type 'a options_fun := 'a options_funUse to update any option in a t, using the optional arguments of options_fun. E.g. set opts ?option1:1 ?option4:"x" ()
Same as set, but passes the result to a continuation, allowing argument stacking
The global reference containing the currently set library options. Access using OpamXxxConfig.(!r.field).
Updates the currently set options in r according to the optional arguments
Sets the options, reading the environment to get default values when unspecified
Sets the options like init, but returns the given value (for arguments stacking)