package opam-repository

  1. Overview
  2. Docs

Module OpamRepositoryRoot.DirSource

Repository root implemented as a directory

Sourcetype t
Sourceval of_dir : OpamTypes.dirname -> t
Sourceval to_dir : t -> OpamTypes.dirname
Sourceval to_string : t -> string
Sourceval quarantine : t -> t

quarantine dir returns a path to a temporary directory dedicated to the original repository root. The returned directory is not created and points to a statically known directory in the same parent directory as dir.

Sourceval with_tmp : (t -> 'a) -> 'a
Sourceval backup : inn:OpamTypes.dirname -> t -> t
Sourceval cwd : unit -> t
Sourceval exists : t -> bool
Sourceval remove : t -> unit
Sourceval move : src:t -> dst:t -> unit
Sourceval copy : src:t -> dst:t -> unit
Sourceval copy_except_vcs : src:t -> dst:t -> unit
Sourceval make_empty : t -> unit
Sourceval dirs : t -> OpamTypes.dirname list
Sourceval is_empty : t -> bool option
Sourceval dirname : t -> OpamTypes.dirname
Sourcemodule Op : sig ... end