You can search for identifiers within the package.
in-package search v0.2.0
opam-repository
OpamGit
Git repository backend (based on OpamVCS)
OpamVCS
val env : unit -> string array
Returns OpamProcess.default_env + some git specific environment variables used to make git calls more reproducible. Note however that it cannot be used if you need values from the global git configs (e.g. /etc/gitconfig or ~/.gitconfig)
OpamProcess.default_env
module VCS : OpamVCS.VCS
module B : OpamRepositoryBackend.S