Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
OpamRepositoryRootSourceThis module abstract the notion of repository root over its concrete implementation (could be a database, a file, a directory, etc.)
quarantine repo_root returns a temporary repository root dedicated to repo_root. the returned repository is not created on disk and points to a statically known repository located in the same parent directory as repo_root.
val read_file :
(module OpamFile.IO_FILE with type t = 'a) ->
?safe:bool ->
t ->
?filename:'a OpamFile.t ->
string ->
'aval patch :
allow_unclean:bool ->
[ `Patch_file of OpamTypes.filename | `Patch_diffs of Patch.t list ] ->
t ->
(Patch.operation list, exn) resultReturns a pair (exists, f) where exists tells whether the repo file exists in the repository and f reads it
Remove repo_name repository roots, directory and archive, if present
Applies the function in the repository root directory. If repository root is an archive, it uncompress it, applies the function and update archive with the changed directory.