Legend:
Page
Library
Module
Module type
Parameter
Class
Class type
Page
Library
Module
Module type
Parameter
Class
Class type
OpamRepositoryRoot.DirSourceRepository root implemented as a directory
quarantine dir returns a path to a temporary directory dedicated to the original repository root. The returned directory is not created and points to a statically known directory in the same parent directory as dir.
module Path :
OpamRepositoryPath.PATH
with type repo_root = t
and type repo_dirname = OpamTypes.dirname
and type 'a typed_file = 'a OpamFile.t